import logging

mqtt_address = "localhost"
mqtt_port = 1883
mqtt_keepalive = 60
mqtt_username = "eric"
mqtt_password = "eric123"

log_level = logging.DEBUG
log_dir = "D:\\Work\\SDK\\agent-bdi\\tests\guide\\_log"

openai_api_key = 'openai_api_key'
